@@ -9,10 +9,9 @@
 # @stable      yes
 # @parse       (general)    url, title, content
 
+from urllib.parse import urlencode
 from lxml.html import fromstring
-from searx.engines.xpath import extract_text
-from searx.utils import eval_xpath
-from searx.url_utils import urlencode
+from searx.utils import extract_text, eval_xpath
 
 # engine dependent config
 categories = ['general']  # TODO , 'images', 'music', 'videos', 'files'
