@@ -14,13 +14,11 @@
 """
 
 from json import loads
-from searx.engines.xpath import extract_text
-from searx.engines.duckduckgo import (
-    _fetch_supported_languages, supported_languages_url,
-    get_region_code, language_aliases
-)
+from urllib.parse import urlencode
+from searx.exceptions import SearxEngineAPIException
+from searx.engines.duckduckgo import get_region_code
+from searx.engines.duckduckgo import _fetch_supported_languages, supported_languages_url  # NOQA # pylint: disable=unused-import
 from searx.poolrequests import get
-from searx.url_utils import urlencode
 
 # engine dependent config
 categories = ['images']
@@ -35,9 +33,12 @@ site_url = 'https://duckduckgo.com/?{query}&iar=images&iax=1&ia=images'
 
 # run query in site to get vqd number needed for requesting images
 # TODO: find a way to get this number without an extra request (is it a hash of the query?)
-def get_vqd(query):
-    res = get(site_url.format(query=urlencode({'q': query})))
+def get_vqd(query, headers):
+    query_url = site_url.format(query=urlencode({'q': query}))
+    res = get(query_url, headers=headers)
     content = res.text
+    if content.find('vqd=\'') == -1:
+        raise SearxEngineAPIException('Request failed')
     vqd = content[content.find('vqd=\'') + 5:]
     vqd = vqd[:vqd.find('\'')]
     return vqd
@@ -47,7 +48,7 @@ def get_vqd(query):
 def request(query, params):
     # to avoid running actual external requests when testing
     if 'is_test' not in params:
-        vqd = get_vqd(query)
+        vqd = get_vqd(query, params['headers'])
     else:
         vqd = '12345'
 
@@ -56,8 +57,12 @@ def request(query, params):
     safesearch = params['safesearch'] - 1
 
     region_code = get_region_code(params['language'], lang_list=supported_languages)
-    params['url'] = images_url.format(
-        query=urlencode({'q': query, 'l': region_code}), offset=offset, safesearch=safesearch, vqd=vqd)
+    if region_code:
+        params['url'] = images_url.format(
+            query=urlencode({'q': query, 'l': region_code}), offset=offset, safesearch=safesearch, vqd=vqd)
+    else:
+        params['url'] = images_url.format(
+            query=urlencode({'q': query}), offset=offset, safesearch=safesearch, vqd=vqd)
 
     return params
 
@@ -67,10 +72,7 @@ def response(resp):
     results = []
 
     content = resp.text
-    try:
-        res_json = loads(content)
-    except:
-        return []
+    res_json = loads(content)
 
     # parse results
     for result in res_json['results']:
