@@ -9,9 +9,9 @@
  @parse        url, title, content
 """
 
+from urllib.parse import urlencode
 from lxml import html
-from searx.engines.xpath import extract_text
-from searx.url_utils import urlencode
+from searx.utils import extract_text
 
 # engine dependent config
 categories = ['files']
