@@ -10,7 +10,7 @@ Frinkiac (Images)
 """
 
 from json import loads
-from searx.url_utils import urlencode
+from urllib.parse import urlencode
 
 categories = ['images']
 
