@@ -11,8 +11,8 @@
 """
 
 from json import loads
+from urllib.parse import urlencode
 from searx.utils import searx_useragent
-from searx.url_utils import urlencode
 
 # engine dependent config
 categories = ['map']
@@ -21,7 +21,7 @@ language_support = True
 number_of_results = 10
 
 # search-url
-base_url = 'https://photon.komoot.de/'
+base_url = 'https://photon.komoot.io/'
 search_string = 'api/?{query}&limit={limit}'
 result_base_url = 'https://openstreetmap.org/{osm_type}/{osm_id}'
 
