@@ -10,9 +10,7 @@
 
 from functools import reduce
 from json import loads
-from searx.engines.xpath import extract_text
-from searx.utils import list_get
-from searx.url_utils import quote_plus
+from urllib.parse import quote_plus
 
 # engine dependent config
 categories = ['videos', 'music']
@@ -51,7 +49,7 @@ def response(resp):
     results = []
 
     results_data = resp.text[resp.text.find('ytInitialData'):]
-    results_data = results_data[results_data.find('{'):results_data.find(';\n')]
+    results_data = results_data[results_data.find('{'):results_data.find(';</script>')]
 
     results_json = loads(results_data) if results_data else {}
     sections = results_json.get('contents', {})\
@@ -70,11 +68,15 @@ def response(resp):
                 title = get_text_from_json(video.get('title', {}))
                 content = get_text_from_json(video.get('descriptionSnippet', {}))
                 embedded = embedded_url.format(videoid=videoid)
+                author = get_text_from_json(video.get('ownerText', {}))
+                length = get_text_from_json(video.get('lengthText', {}))
 
                 # append result
                 results.append({'url': url,
                                 'title': title,
                                 'content': content,
+                                'author': author,
+                                'length': length,
                                 'template': 'videos.html',
                                 'embedded': embedded,
                                 'thumbnail': thumbnail})
