@@ -1,7 +1,4 @@
-import json
-from os.path import join
-
-from searx import searx_dir
+from searx.data import bangs_loader
 
 # bangs data coming from the following url convert to json with
 # https://raw.githubusercontent.com/jivesearch/jivesearch/master/bangs/bangs.toml
@@ -9,10 +6,9 @@ from searx import searx_dir
 # NOTE only use the get_bang_url
 
 bangs_data = {}
-with open(join(searx_dir, 'data/bangs.json')) as json_file:
-    for bang in json.load(json_file)['bang']:
-        for trigger in bang["triggers"]:
-            bangs_data[trigger] = {x: y for x, y in bang.items() if x != "triggers"}
+for bang in bangs_loader()['bang']:
+    for trigger in bang["triggers"]:
+        bangs_data[trigger] = {x: y for x, y in bang.items() if x != "triggers"}
 
 
 def get_bang_url(search_query):
@@ -23,7 +19,7 @@ def get_bang_url(search_query):
     """
 
     if search_query.external_bang:
-        query = search_query.query.decode('utf-8', 'ignore')
+        query = search_query.query
         bang = _get_bang(search_query.external_bang)
 
         if bang and query:
