@@ -7,11 +7,11 @@ class TestQuery(SearxTestCase):
     def test_simple_query(self):
         query_text = 'the query'
         query = RawTextQuery(query_text, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), query_text)
-        self.assertEquals(len(query.query_parts), 1)
-        self.assertEquals(len(query.languages), 0)
+        self.assertEqual(query.getFullQuery(), query_text)
+        self.assertEqual(len(query.query_parts), 0)
+        self.assertEqual(len(query.user_query_parts), 2)
+        self.assertEqual(len(query.languages), 0)
         self.assertFalse(query.specific)
 
     def test_language_code(self):
@@ -19,11 +19,10 @@ class TestQuery(SearxTestCase):
         query_text = 'the query'
         full_query = ':' + language + ' ' + query_text
         query = RawTextQuery(full_query, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), full_query)
-        self.assertEquals(len(query.query_parts), 3)
-        self.assertEquals(len(query.languages), 1)
+        self.assertEqual(query.getFullQuery(), full_query)
+        self.assertEqual(len(query.query_parts), 1)
+        self.assertEqual(len(query.languages), 1)
         self.assertIn(language, query.languages)
         self.assertFalse(query.specific)
 
@@ -32,10 +31,9 @@ class TestQuery(SearxTestCase):
         query_text = 'the query'
         full_query = ':' + language + ' ' + query_text
         query = RawTextQuery(full_query, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), full_query)
-        self.assertEquals(len(query.query_parts), 3)
+        self.assertEqual(query.getFullQuery(), full_query)
+        self.assertEqual(len(query.query_parts), 1)
         self.assertIn('en', query.languages)
         self.assertFalse(query.specific)
 
@@ -44,10 +42,9 @@ class TestQuery(SearxTestCase):
         query_text = 'the query'
         full_query = ':' + language + ' ' + query_text
         query = RawTextQuery(full_query, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), full_query)
-        self.assertEquals(len(query.query_parts), 3)
+        self.assertEqual(query.getFullQuery(), full_query)
+        self.assertEqual(len(query.query_parts), 1)
         self.assertIn('all', query.languages)
         self.assertFalse(query.specific)
 
@@ -56,51 +53,46 @@ class TestQuery(SearxTestCase):
         query_text = 'the query'
         full_query = ':' + language + ' ' + query_text
         query = RawTextQuery(full_query, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), full_query)
-        self.assertEquals(len(query.query_parts), 1)
-        self.assertEquals(len(query.languages), 0)
+        self.assertEqual(query.getFullQuery(), full_query)
+        self.assertEqual(len(query.query_parts), 0)
+        self.assertEqual(len(query.languages), 0)
         self.assertFalse(query.specific)
 
     def test_timeout_below100(self):
         query_text = '<3 the query'
         query = RawTextQuery(query_text, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), query_text)
-        self.assertEquals(len(query.query_parts), 3)
-        self.assertEquals(query.timeout_limit, 3)
+        self.assertEqual(query.getFullQuery(), query_text)
+        self.assertEqual(len(query.query_parts), 1)
+        self.assertEqual(query.timeout_limit, 3)
         self.assertFalse(query.specific)
 
     def test_timeout_above100(self):
         query_text = '<350 the query'
         query = RawTextQuery(query_text, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), query_text)
-        self.assertEquals(len(query.query_parts), 3)
-        self.assertEquals(query.timeout_limit, 0.35)
+        self.assertEqual(query.getFullQuery(), query_text)
+        self.assertEqual(len(query.query_parts), 1)
+        self.assertEqual(query.timeout_limit, 0.35)
         self.assertFalse(query.specific)
 
     def test_timeout_above1000(self):
         query_text = '<3500 the query'
         query = RawTextQuery(query_text, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), query_text)
-        self.assertEquals(len(query.query_parts), 3)
-        self.assertEquals(query.timeout_limit, 3.5)
+        self.assertEqual(query.getFullQuery(), query_text)
+        self.assertEqual(len(query.query_parts), 1)
+        self.assertEqual(query.timeout_limit, 3.5)
         self.assertFalse(query.specific)
 
     def test_timeout_invalid(self):
         # invalid number: it is not bang but it is part of the query
         query_text = '<xxx the query'
         query = RawTextQuery(query_text, [])
-        query.parse_query()
 
-        self.assertEquals(query.getFullQuery(), query_text)
-        self.assertEquals(len(query.query_parts), 1)
-        self.assertEquals(query.query_parts[0], query_text)
-        self.assertEquals(query.timeout_limit, None)
+        self.assertEqual(query.getFullQuery(), query_text)
+        self.assertEqual(len(query.query_parts), 0)
+        self.assertEqual(query.getQuery(), query_text)
+        self.assertEqual(query.timeout_limit, None)
         self.assertFalse(query.specific)
